% Last modified September 2013 by EA Allen
%--------------------------------------------------------------------------
% Demonstration of GIFT: ICA decomposition of resting-state data.
% Emphasis on command line integration with GUI.
% Intended to accompany gift_demo.pdf
% Dependencies:
%   -GroupICATv3.0a % GIFT software
%--------------------------------------------------------------------------

ROOTDIR =  'C:\Teaching\Grenoble';  % update this line with your own root directory
addpath(genpath(fullfile(ROOTDIR, 'GroupICATv3.0a'))); % add the GIFT package to our path
addpath(genpath(fullfile(ROOTDIR, 'CODE'))); % add OUR code (make sure this goes last)
DATADIR = fullfile(ROOTDIR, 'DATA'); % keep track of the data directory
%--------------------------------------------------------------------------

%% 1. Get ready to run ICA
% Check your data! (garbage in = garbage out)
% Be aware of and discard or correct susceptibility artifacts,
% excessive motion, poor spatial normalization, etc.
%----------------------------------------------------------------------
% Below is an example of how to look at the framewise displacement
% as computed from the realignment parameters.  Just one of the MANY
% summary values that can be used to assess the quality of the data.

% Load in realignment parameters (estimated in preprocessing)
motfiles = rdir(fullfile(DATADIR, '*', 'rest', 'rp*.txt'));
FD = cell(length(motfiles),1);
for ii = 1:length(motfiles)
    
    rp = load(motfiles{ii});
    % rp is [180 TRs x 6 directions (x, y, z, pitch, roll, yaw)]
    
    % convert rotations (radians) into translations (mm)
    rad = 50; % assume a head radius of 50 mm
    rp(:,4:6) = rad*tan(rp(:,4:6));
    
    % compute the FD (Euclidean distance frame-to-frame)
    FD{ii} = sqrt(sum(diff(rp).^2,2));
end

% Take a look at the 'best' and 'worst' movers
meanFD = cellfun(@mean, FD);
[mv, bestIND] = min(meanFD); [mv, worstIND] = max(meanFD);
example_subjects = [bestIND, worstIND];

F1 = figure('Color', 'w', 'Name', 'FD');
for ii = 1:length(example_subjects)
    subplot(length(example_subjects),1,ii)
    plot(FD{example_subjects(ii)}, 'r', 'LineWidth', 2)
    xlabel('TR')
    ylabel('FD (mm)'); box off
    axis tight, set(gca, 'Ylim', [0 2])
    title(motfiles{example_subjects(ii)}, 'Interpreter', 'none')
end

close all, clearvars -except *DIR
%%  Make a mask and check it
% GIFT will produce a mask (based on intensity), but ALWAYS check it!
% You may be missing important part of the brain (Motor cortex,
% cerebellum) or include the eyes (lots of variance!).  It is a good
% idea to make your own mask (can use the Brain Extraction Tool in FSL
% or MRIcroN, 3dAutomask in AFNI).  For this dataset I have used BET to
% extract the brain from the average image across subjects.

maskfile = fullfile(DATADIR, 'mask.nii'); % a generous mask made from this data earlier
% Short demo of checking the mask in MRIcroN
%%
% clean up after ourselves
close all, clearvars -except *DIR
%----------------------------------------------------------------------
%% Determine appropriate model order (or set to high value)
% GIFT will estimate the number of components in each dataset using
% information theoretic criteria (ITC), then select a model order for the
% group as the max of the MDL estimates across datasets.
% I like to compute the number of components for each dataset ahead of
% time and take a look at the distributions, then decide how to proceed.
% Of course, ITC are theoretical, and they rest on assumptions about the
% independence of the measurements that are not consistent with the
% data.  The ITC in GIFT have been adapted from original formulations to deal
% with sample dependence, but nothing is perfect. Empirical (not theoretical)
% that consider that stability of components have also been suggested.
% Instead of using ITC, we can initially set the model order to a high number (e.g., 100)
% and assess the stability of resulting components, then revise (decrease)
% the model order from there. This method isn't perfect either.
%----------------------------------------------------------------------
% Below is an example of how to estimate the number of components for
% several datasets using ITC. We'll take a look at empirical methods later.

% list of resting-state data files
fnames = rdir(fullfile(DATADIR,'*', 'rest', 's5war*180.nii'));
% a generous mask made from this data earlier
maskfile = fullfile(DATADIR, 'mask.nii');

% For the first subject:
[comp_est, mdl, aic, kic] = icatb_estimate_dimension(fnames{1}, maskfile);
% number of components is the minimum of the curve
[mv,mdl_mind]=min(mdl); [mv,aic_mind]=min(aic); [mv,kic_mind]=min(kic);

F1 = figure('Color', 'w', 'Name', 'ITCsinglesubject');
plot(1:length(mdl), [mdl; aic; kic])
legend('Minimum description length', 'Akaike information criterion', 'KullbackLeibler information criterion')
xlabel('Number of components')
ylabel('Criterion'); box off
title(sprintf('MDL: %d, AIC: %d, KIC; %d', mdl_mind, aic_mind, kic_mind));

% All the subjects. Don't do this now unless you are very patient or have a very fast computer.
mdl_all = zeros(1,length(fnames));
aic_all = zeros(1,length(fnames));
kic_all = zeros(1,length(fnames));
for ii = 1:length(fnames)
    fprintf('Working on subject %d of %d\n', ii, length(fnames))
    [comp_est, mdl, aic, kic] = icatb_estimate_dimension(fnames{ii}, maskfile);
    [mv,mdl_all(ii)]=min(mdl); [mv,aic_all(ii)]=min(aic); [mv,kic_all(ii)]=min(kic);
end

F2 = figure('Color', 'w', 'Name', 'ITCdistributions');
subplot(1,3,1); hist(mdl_all); box off; hold on; plot(mean(mdl_all), 6, 'rv')
xlabel('Number of Comps'); ylabel('Frequency'); title('MDL'); axis tight;
subplot(1,3,2); hist(aic_all); box off; hold on; plot(mean(aic_all), 6, 'rv')
xlabel('Number of Comps'); ylabel('Frequency'); title('AIC'); axis tight;
subplot(1,3,3); hist(kic_all); box off; hold on; plot(mean(kic_all), 6, 'rv')
xlabel('Number of Comps'); ylabel('Frequency'); title('KIC'); axis tight;
%% 2. Run ICA!
% Setup an analysis in GIFT using GUI.
% Just type 'gift' at the command line.

% Alternatively, let's set it up in a batch script.
edit('gift_demo_batchfile.m')
% The file is basically a long list of variable declarations.
% We'll only discuss some of the variables since many of them are
% irrelevant for today's demo, but please ask questions about anything
% that doesn't make sense or that seems odd.

% Before running anything, make sure to check your ICAtb defaults:
which('icatb_defaults')

% So how do we run the ICA from a batch file?
% This line of code will do all the steps:
icatb_batch_file_run('gift_demo_batchfile');

% let's run each of these steps at a time:
%---------------------------------------------------------------------
% Read the parameters from the batch script:
param_file = icatb_setup_analysis('gift_demo_batchfile');
% Store the parameters properly:
sesInfo = icatb_runAnalysis(sesInfo, 2);
% take a look at 'sesInfo'.  Is everything correct?

% Data reduction:
% Runs both single-subject PCA and the group level PCA
sesInfo = icatb_runAnalysis(sesInfo, 3);

% calculate ICA:
sesInfo = icatb_runAnalysis(sesInfo, 4);

%%  Initial output
% Take some time explore the output files. Do you know what each one
% represents?

% Take a look at the ICASSO output, particularly the Iq plot.
% Is there a clear "knee" in the plot where the cluster quality drops?
% If these plots aren't clear, you can load in the Iq values:
A = load(fullfile(ROOTDIR,'ICA','rest_icasso_results.mat'));
F3 = figure('Color', 'w', 'Name', 'ICASSOIq');
plot(sort(A.iq, 'descend'), 'r.-'); axis tight; box off;
xlabel('Components')
ylabel('Cluster Quality Index')

%%  Revise the model order?
% Based on the ICASSO results, we can revise our estimate of model order
% and re-run parts of the analyis.

% Update the number of components (e.g., try a lower model order)
sesInfo.userInput.numOfPC2 = 70;

% Update some ICA options
nCind = find(strcmp(sesInfo.userInput.ICA_Options, 'ncomps'));
sesInfo.userInput.ICA_Options{nCind + 1} = sesInfo.userInput.numOfPC2;

% Store the parameters correctly
sesInfo = icatb_runAnalysis(sesInfo, 2);

% We don't need to rerun subject PCAs, just the group:
sesInfo.reductionStepsToRun = 2;
sesInfo = icatb_runAnalysis(sesInfo, 3);

% calculate ICA
sesInfo = icatb_runAnalysis(sesInfo, 4);

%% Final steps
% If we're satisfied with the model order, we can go ahead and run the
% rest of the steps.

% back-reconstruction
% estimation of components and timecourses for each subject.
sesInfo = icatb_runAnalysis(sesInfo, 5);

% calibration (scale components)
sesInfo = icatb_runAnalysis(sesInfo, 6);

% group stats
sesInfo = icatb_runAnalysis(sesInfo, 7);

%% 3. Component visualization and selection
% Some very nice tools very visualization are available through the
% Display GUI.
% Alternatively, you can open the .img or .nii files in your
% preferred visualization program (e.g., MRIcroN, AFNI, etc.) that
% allows interactive use, overlays, and atlas integration.

% Depending on how much time is left, we'll play around with GIFT's
% visualization GUI, and also try out some visualizations with the
% newly added MANCOVAN toolbox which shows both the spatial maps and
% component spectra, which can be very useful for determining whether
% components are of interest (particularly in resting-state studies
% where task-relatedness cannot be computed).

% Figures can also be produced without the GUI in an automated fashion:
icatb_component_viewer(param_file, ...
    'TR', 3.34, ...
    'comps', 1:5, ...
    'image_values', 'Positive and Negative', ...
    'convert_to_zscores', 1, ...
    'threshold', 4', ...
    'freq_limits', [.09 .1], ...
    'anatomical_file', fullfile(ROOTDIR, 'GroupICATv3.0a', 'icatb', 'icatb_templates', 'ch2bet.nii'));
